package com.simuwang.manage.api.product;

import com.simuwang.base.ano.ExcludeGlobalResult;
import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.common.util.EncodeUtil;
import com.simuwang.base.common.util.ExcelUtil;
import com.simuwang.base.pojo.dto.ExcelDeletionInfoDTO;
import com.simuwang.base.pojo.dto.query.ProductPageQuery;
import com.simuwang.base.pojo.vo.*;
import com.simuwang.logging.SystemLog;
import com.simuwang.manage.service.ProductService;
import com.simuwang.manage.service.impl.DistributionServiceImpl;
import com.simuwang.manage.service.impl.ProductServiceImpl;
import com.smppw.common.pojo.ResultVo;
import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletResponse;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@SystemLog(value = "产品要素管理")
@RestController
@RequestMapping("/v1/product")
public class ProductController {
    @Autowired
    public ProductService productService;


    private static final Logger logger = LoggerFactory.getLogger(ProductController.class);
    /**
     * 查询基金要素列表
     * @param prductPageQuery
     * @return
     */
    @SystemLog(value = "查询基金要素列表")
    @GetMapping("/product-list")
    public MybatisPage<ProductInformationVO> searchProductList(ProductPageQuery prductPageQuery){
        MybatisPage<ProductInformationVO> result = productService.searchProductList(prductPageQuery);
        return result;
    }

    /**
     * 上传产品要素
     *
     * @param file
     * @return
     */
    @SystemLog(value = "上传产品要素", type = SystemLog.Type.UPLOAD_OR_IMPORT)
    @PostMapping("upload")
    public ResultVo uploadProduct(@RequestPart(value = "file") MultipartFile file) {
        return productService.uploadProduct(file);
    }

    @SystemLog(value = "删除产品要素", type = SystemLog.Type.UPLOAD_OR_IMPORT)
    @PostMapping("remove")
    public ResultVo removeProduct(@RequestBody IdListVO idListVO) {
        return productService.removeProduct(idListVO.getIdList());
    }

    @SystemLog(value = "查询产品要素明细", type = SystemLog.Type.UPLOAD_OR_IMPORT)
    @GetMapping("/detail")
    public ResultVo productDetail( IdVO idVO) {
        return productService.productDetail(idVO);
    }

    /**
     * 下载数据缺失
     * @param idListVO
     * @return
     */
    @SystemLog(value = "导出产品要素数据", type = SystemLog.Type.DOWNLOAD_OR_EXPORT)
    @ExcludeGlobalResult
    @PostMapping("/export")
    public void exportProductInfomration(@RequestBody IdListVO idListVO, HttpServletResponse response ) {
        List<ProductDataVO> productDataVOList = productService.selectProuctInformationByIdList(idListVO.getIdList());
        XSSFWorkbook wb = ExcelUtil.getProductXSSFWorkbook(productDataVOList);
        try {
            response.setContentType("application/x-xls");
            response.setCharacterEncoding("utf-8");
            response.addHeader("Content-Disposition", "attachment;filename=" + EncodeUtil.encodeUTF8("产品要素.xlsx"));
            ServletOutputStream outputStream = response.getOutputStream();
            wb.write(outputStream);
            outputStream.flush();
            outputStream.close();
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        } finally {
            try {
                wb.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
