package com.simuwang.manage.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.read.listener.PageReadListener;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.simuwang.base.common.conts.ExcelConst;
import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.common.util.DateUtils;
import com.simuwang.base.common.util.EmailUtil;
import com.simuwang.base.common.util.StringUtil;
import com.simuwang.base.mapper.daq.CompanyContactInfoMapper;
import com.simuwang.base.mapper.daq.CompanyEmailSendHistoryMapper;
import com.simuwang.base.mapper.daq.ContactInformationMapper;
import com.simuwang.base.mapper.daq.SendEmailMapper;
import com.simuwang.base.mapper.daq.system.SysConfigMapper;
import com.simuwang.base.pojo.dos.CompanyEmailSendHistoryDO;
import com.simuwang.base.pojo.dos.SendEmailInfoDO;
import com.simuwang.base.pojo.dto.ContactExcelData;
import com.simuwang.base.pojo.dos.ContactInformationDO;
import com.simuwang.base.pojo.dto.MailboxInfoDTO;
import com.simuwang.base.pojo.dto.query.ContactQuery;
import com.simuwang.base.pojo.vo.*;
import com.simuwang.manage.service.CompanyEmailConfigService;
import com.simuwang.manage.service.CompanyInformationService;
import com.simuwang.manage.service.ContactInformationService;
import com.simuwang.shiro.utils.UserUtils;
import com.smppw.common.pojo.ResultVo;
import com.smppw.common.pojo.enums.status.ResultCode;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ContactInformationServiceImpl implements ContactInformationService {
    @Autowired
    private ContactInformationMapper contactInformationMapper;

    @Autowired
    private CompanyContactInfoMapper companyContactInfoMapper;

    @Autowired
    private CompanyInformationService companyInformationService;

    @Autowired
    private CompanyEmailSendHistoryMapper companyEmailSendHistoryMapper;

    @Autowired
    private SysConfigMapper sysConfigMapper;
    @Autowired
    private SendEmailMapper sendEmailMapper;

    @Value("${email.file.path}")
    private String path;
    private static final Logger logger = LoggerFactory.getLogger(ContactInformationServiceImpl.class);

    @Override
    public ResultVo saveContactInformation(List<ContactInformationVO> contactInformationVOList) {
        ResultVo vo = new ResultVo(ResultCode.SAVE_SUCCESS.getCode());
        for (ContactInformationVO contactInformationVO : contactInformationVOList) {
            contactInformationVO.setUpdateTime(DateUtils.getTime());
            contactInformationVO.setIsvalid(1);
            contactInformationVO.setCreatorId(UserUtils.getLoginUser().getUserId());
            contactInformationVO.setUpdaterId(UserUtils.getLoginUser().getUserId());
            ContactInformationDO contactInformationDO = BeanUtil.copyProperties(contactInformationVO, ContactInformationDO.class);
            ContactInformationDO oldContactInformationDO = contactInformationMapper.getContactInformation(contactInformationDO);
            if (oldContactInformationDO != null && contactInformationVO.getId() == null) {
                vo.setData(false);
                vo.setMsg("联系人信息已经存在，无需新增");
                return vo;
            }
            if (oldContactInformationDO == null) {
                contactInformationDO.setCreateTime(new Date());
                contactInformationMapper.insert(contactInformationDO);
            } else {
                contactInformationDO.setId(oldContactInformationDO.getId());
                contactInformationMapper.updateById(contactInformationDO);
            }
        }
        return vo;
    }

    @Override
    public MybatisPage<ContactInformationVO> selectContactInformationList(ContactQuery contactQuery) {
        List<ContactInformationDO> contactInformationDOList = contactInformationMapper.selectContactInformationList(contactQuery);
        long total = contactInformationMapper.countContactInfo(contactQuery);
        List<ContactInformationVO> dataList = contactInformationDOList.stream().map(ContactInformationDO::toVO).collect(Collectors.toList());
        return MybatisPage.of(total,dataList);
    }

    @Override
    public ResultVo deleteById(Integer id) {
        ResultVo vo = new ResultVo(ResultCode.DELETE_SUCCESS.getCode());
        Integer userId = UserUtils.getLoginUser().getUserId();
        //判断联系人是否绑定对接信息，如果有不能删除
        int contactCount = companyContactInfoMapper.selectCountByContactId(id);
        if(contactCount > 0) {
            vo.setCode(ResultCode.DELETE_FAILED.getCode());
            vo.setData(false);
            vo.setMsg("联系人有绑定对接信息，不能删除");
            return vo;
        }
        contactInformationMapper.deleteContactInformationById(id,userId);
        vo.setData(true);
        return vo;
    }

    @Override
    public List<ContactInformationDO> selectContactInformationListByCompanyId(String companyId) {
        return contactInformationMapper.selectContactInformationListByCompanyId(companyId);
    }

    @Override
    public ContactInformationDO selectById(Integer id) {
        return contactInformationMapper.selectById(id);
    }

    @Override
    public ResultVo uploadContactInformation(MultipartFile excelFile) {
        ResultVo vo = new ResultVo(ResultCode.SUCCESS);
        File file = null;
        try{
            InputStream inputStream = excelFile.getInputStream();
            file = new File(path+"/upload/"+ System.currentTimeMillis()+"/"+excelFile.getOriginalFilename());
            FileUtils.copyInputStreamToFile(inputStream,file);
            List<ContactExcelData> list = parseContactFile(file);
            vo.setData(parseResult(list));
            inputStream.close();
        }catch (Exception e){
            logger.error(e.getMessage(),e);
            vo.setMsg("文件解析异常");
            vo.setData(false);
            return vo;
        }
        return vo;
    }

    @Override
    public ResultVo sendEmail(List<MultipartFile> files, String contractId, String emailTitle, String templateCode, String emailBody) {
        ResultVo vo = new ResultVo(ResultCode.SAVE_SUCCESS.getCode());
        SendEmailInfoDO sendEmailInfoDO = new SendEmailInfoDO();
        sendEmailInfoDO.setEmailTitle(emailTitle);
        sendEmailInfoDO.setTemplateCode(templateCode);
        sendEmailInfoDO.setEmailBody(emailBody);
        sendEmailInfoDO.setContractId(Integer.parseInt(contractId));
        sendEmailInfoDO.setCreateTime(new Date());
        sendEmailInfoDO.setUpdateTime(new Date());
        sendEmailInfoDO.setIsvalid(1);
        sendEmailInfoDO.setCreatorId(UserUtils.getLoginUser().getUserId());
        sendEmailInfoDO.setUpdaterId(UserUtils.getLoginUser().getUserId());
        List<File> fileList = new ArrayList<>();
        if(files != null && !files.isEmpty()){
            StringBuilder filePath = new StringBuilder();
            StringBuilder fileName = new StringBuilder();
            for (MultipartFile file : files) {
                InputStream inputStream = null;
                try {
                    inputStream = file.getInputStream();
                    String attachmentFilePath = path+"/upload/"+ System.currentTimeMillis()+"/"+file.getOriginalFilename();
                    File attachmentFile = new File(attachmentFilePath);
                    FileUtils.copyInputStreamToFile(inputStream,attachmentFile);
                    inputStream.close();
                    filePath.append(attachmentFilePath).append(";");
                    fileName.append(file.getOriginalFilename()).append(";");
                    fileList.add(attachmentFile);
                } catch (IOException e) {
                    logger.error(e.getMessage(),e);
                }
            }
            sendEmailInfoDO.setFileName(fileName.toString());
            sendEmailInfoDO.setFilePath(filePath.toString());
        }
        MailboxInfoDTO mailboxInfoDTO = getFromEmailInfo();
        ContactInformationDO contactInformationDO = contactInformationMapper.selectById(contractId);
        vo.setData(true);
        try {
            String host = sysConfigMapper.selectConfigByKey("email.host")==null?"":sysConfigMapper.selectConfigByKey("email.host");
            emailBody = emailBody.replaceAll("\r\n","<br/>").replaceAll("\n","<br/>");
            try{
                EmailUtil.senEmail(mailboxInfoDTO,contactInformationDO.getContactEmail(),fileList,emailBody,host,emailTitle);
            }catch (Exception e){
                logger.error(e.getMessage(),e);
                EmailUtil.senEmail(mailboxInfoDTO,contactInformationDO.getContactEmail(),fileList,emailBody,host,emailTitle);
            }
            sendEmailMapper.insert(sendEmailInfoDO);
            saveCompanyEmailSendHistory(contactInformationDO.getCompanyId(),contactInformationDO.getContactEmail(),emailTitle,1, com.simuwang.base.common.enums.ResultCode.SEND_SUCCESS.getMsg());
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            vo.setData(false);
            vo.setMsg(e.getMessage());
            saveCompanyEmailSendHistory(contactInformationDO.getCompanyId(),contactInformationDO.getContactEmail(),emailTitle,0, e.getMessage());
        }
        return vo;
    }

    @Override
    public List<ContactInformationDO> selectByIdList(List<Integer> idList) {
        return contactInformationMapper.selectIdList(idList);
    }


    private MailboxInfoDTO getFromEmailInfo(){
        MailboxInfoDTO dto = new MailboxInfoDTO();
        dto.setAccount(sysConfigMapper.selectConfigByKey("sender.email"));
        dto.setProtocol(sysConfigMapper.selectConfigByKey("sender.protocol"));
        dto.setPassword(sysConfigMapper.selectConfigByKey("sender.password"));
        dto.setPort(sysConfigMapper.selectConfigByKey("sender.port"));
        dto.setHost(sysConfigMapper.selectConfigByKey("sender.host"));
        dto.setSsl(sysConfigMapper.selectConfigByKey("sender.ssl"));
        return dto;
    }

    private void saveCompanyEmailSendHistory(String companyId, String email, String emailTitle,Integer sendStatus,String msg) {
        CompanyEmailSendHistoryDO historyDO = new CompanyEmailSendHistoryDO();
        historyDO.setCompanyId(companyId);
        historyDO.setEmail(email);
        historyDO.setSendStatus(sendStatus);
        historyDO.setSendRemark(msg);
        historyDO.setEmailTitle(emailTitle);
        historyDO.setIsvalid(1);
        historyDO.setSendTime(DateUtils.getNowDate());
        historyDO.setCreateTime(DateUtils.getNowDate());
        historyDO.setCreateTime(DateUtils.getNowDate());
        companyEmailSendHistoryMapper.saveCompanyEmailSendHistory(historyDO);
    }

    private boolean saveContactInformationDO(ContactInformationDO contactInformationDO) {
        try{
            ContactInformationDO contactInformation = contactInformationMapper.selectByKey(contactInformationDO);
            Integer userId = UserUtils.getLoginUser().getUserId();
            if(contactInformation != null){
                contactInformationDO.setId(contactInformation.getId());
                contactInformationDO.setIsvalid(1);
                contactInformationDO.setUpdateTime(new Date());
                contactInformationDO.setUpdaterId(userId);
                contactInformationDO.setContactRemark(contactInformation.getContactRemark());
                contactInformationDO.setFileId(contactInformation.getFileId());
                contactInformationMapper.updateById(contactInformationDO);
            }else{
                contactInformationDO.setIsvalid(1);
                contactInformationDO.setUpdateTime(new Date());
                contactInformationDO.setUpdaterId(userId);
                contactInformationDO.setCreateTime(new Date());
                contactInformationDO.setCreatorId(userId);
                contactInformationMapper.insert(contactInformationDO);
            }
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            return false;
        }
        return true;
    }

    private Object parseResult(List<ContactExcelData> list) {
        ContactUploadResult result = new ContactUploadResult();
        int startRow = 1;
        List<ContactExcelSuccessDataVO> successDataVOList = new ArrayList<>();
        List<ContactExcelFailDataVO> excelFailDataVOList = new ArrayList<>();
        for(int dataIdx=0;dataIdx<list.size(); dataIdx++){
            ContactExcelData excelData = list.get(dataIdx);
            try{
                if((StringUtil.isEmpty(excelData.getCompanyId()) || StringUtil.isEmpty(excelData.getContactName()))
                        || StringUtil.isEmpty(excelData.getContactEmail()) || (StringUtil.isEmpty(excelData.getContactPhone()) && StringUtil.isEmpty(excelData.getContactWechat()))
                ){
                    ContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData, ExcelConst.REQUIRE_FIELD,dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                    continue;
                }
                if(StringUtil.isEmpty(companyInformationService.getCompanyNameById(excelData.getCompanyId()))){
                    ContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData, ExcelConst.ERROR_COMPANY_ID,dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                    continue;
                }

                //开始处理成功数据
                ContactInformationDO contactInformationDO = new ContactInformationDO();
                contactInformationDO.setContactName(excelData.getContactName());
                contactInformationDO.setContactEmail(excelData.getContactEmail());
                contactInformationDO.setContactWechat(excelData.getContactWechat());
                contactInformationDO.setContactPosition(excelData.getContactPosition());
                contactInformationDO.setContactPhone(excelData.getContactPhone());
                contactInformationDO.setCompanyId(excelData.getCompanyId());
                boolean saveFlag= saveContactInformationDO(contactInformationDO);
                if(saveFlag){
                    ContactExcelSuccessDataVO successDataVO = new ContactExcelSuccessDataVO();
                    successDataVO.setCompanyId(excelData.getCompanyId());
                    successDataVO.setCompanyName(companyInformationService.getCompanyNameById(excelData.getCompanyId()));
                    successDataVO.setContactPosition(excelData.getContactPosition());
                    successDataVO.setContactPhone(excelData.getContactPhone());
                    successDataVO.setContactName(excelData.getContactName());
                    successDataVO.setContactEmail(excelData.getContactEmail());
                    successDataVO.setContactWechat(excelData.getContactWechat());
                    successDataVOList.add(successDataVO);
                }else{
                    ContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData,"数据入库异常",dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                }
            }catch (Exception e){
                logger.error(e.getMessage(),e);
                ContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData,e.getMessage(),dataIdx+startRow);
                excelFailDataVOList.add(failDataVO);
            }
        }
        result.setSuccess(successDataVOList);
        result.setFail(excelFailDataVOList);
        result.setTotal(successDataVOList.size()+excelFailDataVOList.size());
        result.setFailCount(excelFailDataVOList.size());
        result.setSuccessCount(successDataVOList.size());
        return result;
    }
    private ContactExcelFailDataVO toExcelFailDataVO(ContactExcelData excelData, String msg,Integer rowNum) {
        ContactExcelFailDataVO failDataVO = new ContactExcelFailDataVO();
        failDataVO.setFailReason(msg);
        failDataVO.setRowNum(rowNum);
        failDataVO.setContactEmail(excelData.getContactEmail());
        failDataVO.setContactPhone(excelData.getContactPhone());
        failDataVO.setContactPosition(excelData.getContactPosition());
        failDataVO.setCompanyId(excelData.getCompanyId());
        failDataVO.setContactName(excelData.getContactName());
        failDataVO.setCompanyName(companyInformationService.getCompanyNameById(excelData.getCompanyId()));
        return failDataVO;
    }

    private List<ContactExcelData> parseContactFile(File file) {
        // 创建一个 list 存储每行的数据，即 ExcelData 对象
        List<ContactExcelData> list = new ArrayList<>();
        // 直接使用 EasyExcel 的 read 方法，同时定义表头的类型，以便将列中数据映射为 ExcelData 对象
        EasyExcel.read(file, ContactExcelData.class, new PageReadListener<ContactExcelData>(dataList -> {
            // 并且每行数据，并将其 add 至 list 中
            for (ContactExcelData excelData : dataList) {
                if (excelData != null) {
                    list.add(excelData);
                }
            }
        })).excelType(ExcelTypeEnum.XLSX).sheet().doRead();
        return list;
    }
}
